Drupal.uv = Drupal.uv || {};

(function ($) {
  Drupal.behaviors.uv = {
    attach: function () {
      var fieldNames = Drupal.settings.uv.fields_to_validate;
      var $fields = $(fieldNames.map(function(name) {return '.field-name-'+name;}).join(', '));
      $fields.each(function() {
        var $field = $(this);
        var $input = $('input', $field);
        // Check if listener already exists.
        if($.data($input[0], 'events').keyup.some(function(listener) { return listener.namespace === 'uv';})) {
          // Continue.
          return true;
        }
        var fieldName = $field.attr("class").match(/field-name-([^ ]*)/)[1];
        var fieldData = $field.data();
        $('input', $field).bind('keyup.uv', debounce(function() {
          var $input = $(this);
          var state = {type: 'status'};
          if(fieldData.uv) {
            state = window['Drupal']['uv'][fieldData.uv](fieldName, $input.val());
            if(state.type === 'error') {
              changeInputState($input, state)
            }
          }
          if(state.type !== 'error') {
            $.getJSON(
              '/uv/validate',
              {
                'field': fieldName,
                'value': $input.val(),
                'entity_type': Drupal.settings.uv.entity_type,
                'bundle': Drupal.settings.uv.bundle,
              },
              function (result) {
                if (!result) {
                  console.error(new Error('Drupal Universal Validation module - No field validation result from server.'));
                  return;
                }
                if(result.state === 'failure') {
                  console.error(new Error('Drupal Universal Validation module - '+result.message));
                  return;
                }

                changeInputState($input, result);
              }
            );
          }
        }, 600, false));
      });
      function debounce(func, wait, immediate) {
        var timeout;
        return function() {
          var context = this, args = arguments;
          var later = function() {
            timeout = null;
            if (!immediate) func.apply(context, args);
          };
          var callNow = immediate && !timeout;
          clearTimeout(timeout);
          timeout = setTimeout(later, wait);
          if (callNow) func.apply(context, args);
        };
      };
      function changeInputState($input, state) {
        $input.removeClass('error state warning');
        $input.addClass(state.type);
        var message_element = '<div class="uv-message messages '+state.type+' '+state.info+'">'+state.message+'</div>';
        var $messages = $input.siblings('.uv-message');
        if($messages.length) {
          $messages.replaceWith(message_element);
        }
        else {
          var $fieldSuffix = $input.siblings('.field-suffix');
          var $insertPoint = $fieldSuffix.length ? $fieldSuffix : $input;
          $insertPoint.after(message_element);
        }
      }
    }
  };
})(jQuery);